@@ -1,6 +1,7 @@
 import asyncio
 import logging
 import signal
+import sys
 from functools import partial
 from typing import Union, List, Callable, Tuple
 
@@ -10,14 +11,8 @@ from serial_asyncio import open_serial_connection
 
 from genki_wave.callbacks import WaveCallback
 from genki_wave.constants import API_CHAR_UUID, BAUDRATE
-from genki_wave.data.organization import (
-    ButtonAction,
-    ButtonEvent,
-    ButtonId,
-    DataPackage,
-)
 from genki_wave.data.writing import get_start_api_package
-from genki_wave.protocols import ProtocolAsyncio, ProtocolThread
+from genki_wave.protocols import ProtocolAsyncio, ProtocolThread, CommunicateCancel
 from genki_wave.utils import get_serial_port, get_or_create_event_loop
 
 logging.basicConfig(format="%(levelname).4s:%(asctime)s [%(filename)s:%(lineno)d] - %(message)s ")
@@ -25,22 +20,6 @@ logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
 
 
-class CommunicateCancel:
-    """
-    Class that handles how to cancel asyncio loops with a button press and how to communicate it. Usually defined as
-    a global and sends messages to each part of the script
-    """
-
-    cancel = False
-
-    @staticmethod
-    def is_cancel(button_event: Union[ButtonEvent, DataPackage]) -> bool:
-        """Checks for a hard coded cancel event"""
-        if isinstance(button_event, DataPackage):
-            return False
-        return button_event.button_id == ButtonId.TOP and button_event.action == ButtonAction.EXTRALONG
-
-
 def prepare_protocol_as_bleak_callback_asyncio(protocol: ProtocolAsyncio) -> Callable:
     async def _inner(sender: str, data: bytearray) -> None:
         # NOTE: `bleak` expects a function with this signature
@@ -73,6 +52,15 @@ def bleak_callback(protocol: ProtocolAsyncio) -> Callable:
     return callback
 
 
+def make_disconnect_callback(comm: CommunicateCancel):
+    def cb(client):
+        if not comm.cancel:
+            print(f"Client {client.address} disconnected unexpectedly, exiting")
+            sys.exit(1)
+
+    return cb
+
+
 async def producer_bluetooth(
     protocol: Union[ProtocolAsyncio, ProtocolThread],
     comm: CommunicateCancel,
@@ -92,7 +80,7 @@ async def producer_bluetooth(
     """
     print(f"Connecting to wave at address {ble_address}")
     callback = bleak_callback(protocol)
-    async with BleakClient(ble_address) as client:
+    async with BleakClient(ble_address, disconnected_callback=make_disconnect_callback(comm)) as client:
         await client.start_notify(API_CHAR_UUID, callback)
         await client.write_gatt_char(API_CHAR_UUID, get_start_api_package(), False)
 
